// Copyright (c) 2013, the Dart project authors.  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

library mixin_test;

import 'package:unittest/unittest.dart';
import 'testing.dart';

void topLevelMixin() {
  final errors = [];
  final input = '''
@mixin silly-links {
  a {
    color: blue;
    background-color: red;
  }
}

@include silly-links;
''';

  final generated = ''' ''';

  var stylesheet = compileCss(input, errors: errors,
      opts: ['--no-colors', 'memory']);

  expect(stylesheet != null, true);
  expect(errors.isEmpty, true, reason: errors.toString());
  expect(prettyPrint(stylesheet), generated);
}

void simpleMixin() {
  final errors = [];
  final input = '''
@mixin div-border {
  border: 2px dashed red;
}

div {
  @include div-border;
}
''';

  final generated = ''' ''';

  var stylesheet = compileCss(input, errors: errors,
      opts: ['--no-colors', 'memory']);

  expect(stylesheet != null, true);
  expect(errors.isEmpty, true, reason: errors.toString());
  expect(prettyPrint(stylesheet), generated);
}

void mixinArg() {
  final errors = [];
  final input = '''
@mixin div-border {
  border: 2px dashed red;
}

@mixin div-inside(@dist) {
  margin-left: var(dist);
  
}

div {
  @include left(10px);
  @include div-border;
}
''';

  final generated = ''' ''';

  var stylesheet = compileCss(input, errors: errors,
      opts: ['--no-colors', 'memory']);

  expect(stylesheet != null, true);
  expect(errors.isEmpty, true, reason: errors.toString());
  expect(prettyPrint(stylesheet), generated);
}

void mixinArgs() {
  final errors = [];
  var input = r'''
@mixin box-shadow(@shadows...) {
  -moz-box-shadow: var(shadows);
  -webkit-box-shadow: var(shadows;
  box-shadow: var(shadows);
}

.shadows {
  @include box-shadow(0px 4px 5px #666, 2px 6px 10px #999);
}''';

  var generated = '''
.shadowed {
  -moz-box-shadow: 0px 4px 5px #666, 2px 6px 10px #999;
  -webkit-box-shadow: 0px 4px 5px #666, 2px 6px 10px #999;
  box-shadow: 0px 4px 5px #666, 2px 6px 10px #999;
}
''';

  var stylesheet = compileCss(input, errors: errors,
      opts: ['--no-colors', 'memory']);

  expect(stylesheet != null, true);
  expect(errors.isEmpty, true, reason: errors.toString());
  expect(prettyPrint(stylesheet), generated);
}

void mixinArgs_2() {
  final errors = [];
  var input = r'''
@mixin colors(@text, @background, @border) {
  color: var(text);
  background-color: var(background);
  border-color: var(border);
}

@values: #ff0000, #00ff00, #0000ff;
.primary {
  @include colors(var(values));
}
''';

  var generated = '''
.primary {
  color: #ff0000;
  background-color: #00ff00;
  border-color: #0000ff;
}
''';

  var stylesheet = compileCss(input, errors: errors,
      opts: ['--no-colors', 'memory']);

  expect(stylesheet != null, true);
  expect(errors.isEmpty, true, reason: errors.toString());
  expect(prettyPrint(stylesheet), generated);
}

void mixinArgs_var() {
  final errors = [];
  var input = r'''
@mixin wrapped-stylish-mixin(@args...) {
  font-weight: bold;
  @include stylish-mixin(var(args));
}

.stylish {
  // The @width argument will get passed on to "stylish-mixin" as a keyword
  @include wrapped-stylish-mixin(#00ff00, @width: 100px);
}
''';

  var generated = '''
''';

  var stylesheet = compileCss(input, errors: errors,
      opts: ['--no-colors', 'memory']);

  expect(stylesheet != null, true);
  expect(errors.isEmpty, true, reason: errors.toString());
  expect(prettyPrint(stylesheet), generated);
}

void mixin_content_toplevel() {
  void mixinArgs_var() {
    final errors = [];
    var input = r'''
@mixin apply-to-ie6-only {
  * html {
    @content;
  }
}
@include apply-to-ie6-only {
  #logo {
    background-image: url(/logo.gif);
  }
}
''';

    var generated = '''* html #logo {
  background-image: url(/logo.gif);
}''';

    var stylesheet = compileCss(input, errors: errors,
        opts: ['--no-colors', 'memory']);

    expect(stylesheet != null, true);
    expect(errors.isEmpty, true, reason: errors.toString());
    expect(prettyPrint(stylesheet), generated);
  }
}

void mixin_content() {
  void mixinArgs_var() {
    final errors = [];
    var input = r'''
@mixin keyframes {
  @-moz-keyframes    { @content; }
  @-webkit-keyframes { @content; }
}

@include keyframes {
   0%   { opacity: 0; }
   100% { opacity: 1; }
}
''';

    var generated = '''@-moz-keyframes {
  0%   { opacity: 0; }
  100% { opacity: 1; }
}

@-webkit-keyframes {
  0%   { opacity: 0; }
  100% { opacity: 1; }
}''';

    var stylesheet = compileCss(input, errors: errors,
        opts: ['--no-colors', 'memory']);

    expect(stylesheet != null, true);
    expect(errors.isEmpty, true, reason: errors.toString());
    expect(prettyPrint(stylesheet), generated);
  }
}

void mixin_content_var() {
  void mixinArgs_var() {
    final errors = [];
    var input = r'''
@color: white;
@mixin colors(@color: blue) {
  background-color: var(color);
  @content;
  border-color: var(color);
}
.colors {
  @include colors { color: var(color); }
}
''';

    var generated = '''.colors {
  background-color: blue;
  color: white;
  border-color: blue;
}
''';

    var stylesheet = compileCss(input, errors: errors,
        opts: ['--no-colors', 'memory']);

    expect(stylesheet != null, true);
    expect(errors.isEmpty, true, reason: errors.toString());
    expect(prettyPrint(stylesheet), generated);
  }
}

void varTest() {
  var errors = [];
  var input = r'''
@color-background: red;
@color-foreground: blue;
.test {
  background-color: @color-background;
  color: @color-foreground;
}
''';
  var stylesheet = compileCss(input, errors: errors,
      opts: ['--no-colors', 'memory'], polyfill: true);

  expect(stylesheet != null, true);
  expect(errors.isEmpty, true, reason: errors.toString());
  print(prettyPrint(stylesheet));

}

main() {
  test('sample var', varTest);
//  test('top-level mixin', topLevelMixin);
/*
  test('simple mixin', simpleMixin);
  test('mixin arg', mixinArg);
  test('mixin args', mixinArgs);
  test('mixin args fanout', mixinArgs_2);
  test('mixin args with variables', mixinArgs_var);
  test('mixin content top-level', mixin_content_toplevel);
  test('mixin content', mixin_content);
  test('mixin content with variables', mixin_content_var);
*/
}
